/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator.operation;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode;
import eu.etaxonomy.cdm.model.taxon.IllegalAncestryException;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPersistentPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;

/**
 * Change the taxonomic parent of a given taxon.
 *
 * @author n.hoffmann
 * @created 16.01.2009
 * @version 1.0
 */
public class MoveTaxonOperation extends AbstractPersistentPostOperation {

	/**
	 * A reference to the new taxonomical parent.
	 */
	private ITaxonTreeNode newParentTreeNode;
	/**
	 * A reference to the former taxonomical parents
	 */
	private Map<TaxonNode, ITaxonTreeNode> oldParentTreeNodes;

	private Set<TaxonNode> taxonNodes;

	/**
	 * <p>Constructor for MoveTaxonOperation.</p>
	 *
	 * @param label a {@link java.lang.String} object.
	 * @param undoContext a {@link org.eclipse.core.commands.operations.IUndoContext} object.
	 * @param taxonNodes a {@link java.util.Set} object.
	 * @param newParentTreeNode a {@link eu.etaxonomy.cdm.model.taxon.ITaxonTreeNode} object.
	 * @param postOperationEnabled a {@link eu.etaxonomy.taxeditor.operation.IPostOperationEnabled} object.
	 * @param conversationEnabled a {@link eu.etaxonomy.cdm.api.conversation.IConversationEnabled} object.
	 */
	public MoveTaxonOperation(String label, IUndoContext undoContext,
			Set<TaxonNode> taxonNodes, ITaxonTreeNode newParentTreeNode, IPostOperationEnabled postOperationEnabled, IConversationEnabled conversationEnabled) {
		super(label, undoContext, postOperationEnabled, conversationEnabled);

		this.taxonNodes = taxonNodes;

		this.newParentTreeNode = newParentTreeNode;

		// Save old parent ITaxonTreeNodes for undo
		oldParentTreeNodes = new HashMap<TaxonNode, ITaxonTreeNode>();
		for(TaxonNode taxonNode : taxonNodes){
			this.oldParentTreeNodes.put(taxonNode, taxonNode.getParent());
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		bind();
		monitor.worked(20);

		try {
			for (TaxonNode taxonNode : taxonNodes){
				TaxonNode newTaxonNode = newParentTreeNode.addChildNode(taxonNode,
						newParentTreeNode.getReference(), newParentTreeNode.getMicroReference());
				taxonNodes.add(newTaxonNode);
				monitor.worked(2);
			}
		} catch(IllegalAncestryException e) {
			MessagingUtils.warningDialog("Illegal ancestry", this, e.getMessage());
		}
		monitor.worked(40);

		return postExecute(null);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		return execute(monitor, info);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	/** {@inheritDoc} */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		MessagingUtils.warn(this.getClass(), "Not implemented yet.");

		// iterate over oldParentTreeNodes, delete each TaxonNode from its actual parent and add to its former parent

		return Status.OK_STATUS;
	}
}
